@@ -40,13 +40,27 @@ except ImportError:
     # Can be ImportError or ModuleNotFoundError depending on the Python version,
     # but ModuleNotFoundError is a child of ImportError and is still caught.
     BLOSC_PRESENT = False
-    logger.info( '`blosc` meta-compression library not found, file compression disabled.' )
+    logger.info('`blosc` meta-compression library not found, file compression disabled.')
     DEFAULT_N_THREADS = 1
 try: 
     import rapidjson as json
 except ImportError:
     import json
-    logger.info( '`python-rapidjson` not found, using builtin `json` instead.' )
+    logger.info('`python-rapidjson` not found, using builtin `json` instead.')
+
+def _defaultMetaSerialize(value):
+    """
+    Is called by `json.dumps()` whenever it encounters an object it does 
+    not know how to serialize.
+    """
+    if hasattr(value, '__array_interface__'):
+        # Checking for '__array_interface__' also sanitizes numpy scalars
+        # like np.float32 or np.int32
+        return value.tolist()
+    else:
+        raise TypeError('Unhandled type for JSON serialization: {}'.format(type(value)))
+
+# Do we also want to convert long lists to np.ndarrays?
 
 # Buffer for file I/O
 # Quite arbitrary, in bytes (hand-optimized)
@@ -429,7 +443,7 @@ def readBloscHeader(filehandle):
     return ([nbytes, blocksize, ctbytes], [version, versionlz, flags, typesize])
     
     
-def readMRCHeader(MRCfilename, slices, endian='le', fileConvention = 'ccpem', pixelunits=u'\\AA'):
+def readMRCHeader(MRCfilename, slices=None, endian='le', fileConvention = 'ccpem', pixelunits=u'\\AA'):
     '''
     Reads in the first 1024 bytes from an MRC file and parses it into a Python 
     dictionary, yielding header information. This function is not intended to be 
@@ -501,6 +515,7 @@ def readMRCHeader(MRCfilename, slices, endian='le', fileConvention = 'ccpem', pi
         header['dtype'] = endchar + header['dtype']
 
         # slices is z-axis per frame for list-of-arrays representation
+
         if slices is None:
             # We had a bug in version <= 0.4.1 where we wrote the dimensions 
             # into both (Nx, Ny, Nz) AND (Mx, My, Mz), therefore the slicing 
@@ -508,7 +523,7 @@ def readMRCHeader(MRCfilename, slices, endian='le', fileConvention = 'ccpem', pi
             # check where we force slices to be 1 (i.e. we assume it is a 
             # stack of 2D images).
             if mrcz_version is not None and mrcz_version < StrictVersion('0.5.0'):
-                logger.warning('MRCZ version < 0.5.0, assuming slices == 1.')
+                logger.warning('MRCZ version < 0.5.0 for file {}, assuming slices == 1.'.format(MRCfilename))
                 slices = 1
             else:
                 f.seek(36)
@@ -799,11 +814,11 @@ def writeMRC(input_image, MRCfilename, meta=None, endian='le', dtype=None,
     else: # We are going to append to an already existing file:
         # So we try to figure out its header with 'CCPEM' or 'eman2' file conventions:
         try:
-            header, slices = readMRCHeader(MRCfilename, endian, fileConvention='CCPEM', pixelunits=pixelunits)
+            header, slices = readMRCHeader(MRCfilename, slices=None, endian=endian, fileConvention='CCPEM', pixelunits=pixelunits)
 
         except ValueError:
             try:
-                header, slices = readMRCHeader(MRCfilename, endian, fileConvention='eman2', pixelunits=pixelunits)
+                header, slices = readMRCHeader(MRCfilename, slices=None, endian=endian, fileConvention='eman2', pixelunits=pixelunits)
             except ValueError:
             # If neither 'CCPEM' nor 'eman2' formats satisfy:
                 raise ValueError('Error: unrecognized MRC type for file: %s ' % MRCfilename)
@@ -845,6 +860,7 @@ def writeMRC(input_image, MRCfilename, meta=None, endian='le', dtype=None,
 
     else:
         offset = 0
+
         
     __MRCExport(input_image, header, MRCfilename, slices, 
                 endchar=endchar, offset=offset, idxnewfile=idxnewfile)
@@ -1011,6 +1027,7 @@ def writeMRCHeader(f, header, slices, endchar='<'):
     # Print MX, MY, MZ, the sampling. We only allow for slicing along the z-axis,
     # e.g. for multi-channel STEM.
     f.seek(36)
+
     np.int32(slices).astype(dtype_i4).tofile(f)
 
     # Print cellsize = pixelsize * dimensions
@@ -1089,25 +1106,7 @@ def writeMRCHeader(f, header, slices, endchar='<'):
     
     # Extended header, if meta is not None
     if isinstance(header['meta'], dict):
-        # Encode metadata as bytes with UTF-8
-        for key, value in header['meta'].items():
-            # Sanitize types that `python-rapidjson` and `json` don't understand.
-            # Should we make a copy of `meta` in this case?  We are modifying 
-            # the passed data.
-            if hasattr(value, '__array_interface__'):
-                # Checking for '__array_interface__' also sanitises numpy scalars
-                # like np.float32 or np.int32
-                header['meta'][key] = value.tolist()
-
-        try:
-            jsonMeta = json.dumps(header['meta']).encode('utf-8')
-        except TypeError as e:
-            logger.error('Meta types:')
-            for key, value in header['meta'].items():
-                logger.error('    {}: {}'.format(key, type(value)))
-                if hasattr(value, '__len__'):
-                    logger.error('        ' + str([type(thing) for thing in value]))
-            raise TypeError('Some value in the meta-data is not JSONable; typically this is a buried NumPy array')
+        jsonMeta = json.dumps(header['meta'], default=_defaultMetaSerialize).encode('utf-8')
 
         jsonLen = len(jsonMeta)
         # Length of extended header
